%------------------------------------------------------------%
%--- License terms ------------------------------------------%
%------------------------------------------------------------%

%--- This software is distributed under CC0 1.0 Public Domain 
%--- Dedication. To the extent possible under law, the authors
%--- (Marcus Wagner et al.) have dedicated all copyright and 
%--- related and neighboring rights to this software to the 
%--- public domain worldwide. This software is distributed 
%--- without any warranty. You should have received a copy of the 
%--- CC0 1.0 Universal Public Domain Dedication along with this 
%--- software. If not, write to Creative Commons, PO Box 1866, 
%--- Mountain View, CA 94042, USA.

%------------------------------------------------------------%
%--- General assumptions ------------------------------------%
%------------------------------------------------------------%

clear all;

%--- Execution of the master procedure requires that
%--- a) MATLAB Image Processing Toolbox is available, 
%--- b) the working directory contains three greyscale image 
%---    files, representing the CD14/488nm, CD163/555nm and 
%---    DAPI channel of the capture, respectively;
%--- c) the working directory contains further the 
%---    subprocedures
%---    rof_segm_public_step_01_parameters.m
%---    rof_segm_public_step_02_filename.m
%---    rof_segm_public_step_03a_dapimask.m
%---    rof_segm_public_step_03b_dapiestim.m
%---    rof_segm_public_step_04_channel_488.m
%---    rof_segm_public_step_05_channel_555.m
%---    rof_segm_public_step_06_double.m
%---    tgv1_l2_2D_pd.m

%--- For batch processing, please change this file 
%--- in appropriate way.


display(' ');
display('==== Run of ROF filter based segmentation method (S1) ====');
display(' ');

%------------------------------------------------------------%
%--- Input of images filenames ------------------------------%
%------------------------------------------------------------%


flag_488 = 0;
while flag_488 == 0,
name_488_ori = input('Enter filename of CD14/488nm channel image: ', 's');
if exist(name_488_ori, 'file') == 2, flag_488 = 1; end
display(' ');
end

flag_555 = 0;
while flag_555 == 0,
name_555_ori = input('Enter filename of CD163/555nm channel image: ', 's');
if exist(name_555_ori, 'file') == 2, flag_555 = 1; end
display(' ');
end

flag_dapi = 0;
while flag_dapi == 0,
name_dapi_ori = input('Enter filename of DAPI channel image: ', 's');
if exist(name_dapi_ori, 'file') == 2, flag_dapi = 1; end
display(' ');
end

%------------------------------------------------------------%
%--- Check of parameter use ---------------------------------%
%------------------------------------------------------------%

flag_param = 0;
while flag_param == 0 
defpar = input('Default parameter values will be used. Continue (y/n)? ', 's');
if defpar == 'y' | defpar == 'n', flag_param = 1; end
display(' ');
end

if defpar == 'n'
display('--- (S1) terminates.');
display(' ');
display('Please modify parameters in file "rofsegm_public_step_01_parameters.m"');
display(' ');
return
%--- skip all remaining steps
end

%--- continue with image analysis

%------------------------------------------------------------%
%--- Initialization of the parameters as global variables ---%
%------------------------------------------------------------%

display('--- Initialization of parameters');
display(' ');
rof_segm_public_step_01_parameters;

%------------------------------------------------------------%
%--- Generation of filenames as global variables ------------%
%--- Creation of logfile ------------------------------------%
%------------------------------------------------------------%

display('--- Generation of filenames; creation of logfile');
display(' ');
rof_segm_public_step_02_filenames;

%------------------------------------------------------------%
%--- Generation of cartoon and mask for DAPI channel --------%
%------------------------------------------------------------%

display('--- Generation of cartoon and mask for DAPI channel');
display(' ');
rof_segm_public_step_03a_dapimask;

%------------------------------------------------------------%
%--- Estimation of cell core number within DAPI channel -----%
%------------------------------------------------------------%

display('--- Estimation of cell core numer within DAPI channel');
display(' ');
rof_segm_public_step_03b_dapiestim;

%------------------------------------------------------------%
%--- Segmentation of CD14/488nm channel ---------------------%
%------------------------------------------------------------%

display('--- Macrophage segmentation within CD14/488nm channel');
display(' ');
rof_segm_public_step_04_channel_488;

%------------------------------------------------------------%
%--- Segmentation of CD163/555nm channel --------------------%
%------------------------------------------------------------%

display('--- Macrophage segmentation within CD163/555nm channel');
display(' ');
rof_segm_public_step_05_channel_555;

%------------------------------------------------------------%
%--- Recognition of double stainings ------------------------%
%------------------------------------------------------------%

display('--- Recognition of double-stained macrophages');
display(' ');
rof_segm_public_step_06_double;

display('--- (S1) terminates');
display(' ');

