%------------------------------------------------------------%
%--- License terms ------------------------------------------%
%------------------------------------------------------------%

%--- This software is distributed under CC0 1.0 Public Domain 
%--- Dedication. To the extent possible under law, the authors
%--- (Marcus Wagner et al.) have dedicated all copyright and 
%--- related and neighboring rights to this software to the 
%--- public domain worldwide. This software is distributed 
%--- without any warranty. You should have received a copy of the 
%--- CC0 1.0 Universal Public Domain Dedication along with this 
%--- software. If not, write to Creative Commons, PO Box 1866, 
%--- Mountain View, CA 94042, USA.

%------------------------------------------------------------%
%--- Generation of cartoon and mask for DAPI channel --------%
%------------------------------------------------------------%

%--- assumes that flag_dapi == 1

name_prozedur_dapimask = 'rofsegm_public_step_03a_dapimask.m';
version_prozedur_dapimask = 'version_2019_04_12';

%---------------------------------------------------------- -%
%--- Import of original image -------------------------------%
%------------------------------------------------------------%

I0_dapi = double(imread( name_dapi_ori ))/255;

%------------------------------------------------------------%
%--- Median filtering and TV regularization -----------------%
%------------------------------------------------------------%

I1_dapi = medfilt2(I0_dapi, [medpar_dapi,medpar_dapi], 'symmetric');
%--- median filtering of I0_dapi with medpar_dapi

I2_dapi = I0_dapi - I1_dapi;

clear I0_dapi;
clear I1_dapi;

I2_dapi = I2_dapi .* (I2_dapi >= 0);
%--- cutting of possible negative values

I3_dapi = tgv1_l2_2D_pd(I2_dapi, I2_dapi, regpar_dapi, regit_dapi);
%--- TV regularization, requires function tgv1_l2_2D
display(' ');

clear I2_dapi;

imwrite(I3_dapi, name_dapi_cartoon);
%--- saving of DAPI cartoon

%------------------------------------------------------------%
%--- Generation of raw DAPI mask ----------------------------%
%------------------------------------------------------------%

I4_dapi = (I3_dapi >= schwelle_dapi);
%--- masking of I3_dapi by lower brightness threshold

clear I3_dapi;

dim01 = size(I4_dapi,1); dim02 = size(I4_dapi,2);

I5_dapi = double(zeros(dim01, dim02));
%--- contains the raw DAPI mask

sizehalf = (size_dapi_quadrat - 1) / 2;

for i = (dim01 - sizehalf) : (-1) : (sizehalf + 1), 
for j = (dim02 - sizehalf) : (-1) : (sizehalf + 1)
if I4_dapi (i,j) > 0
I5_dapi( (i - sizehalf ) : (i + sizehalf), (j - sizehalf) : (j + sizehalf)) = ones(size_dapi_quadrat,size_dapi_quadrat);
end, end, end
%--- generates the raw DAPI mask

clear I4_dapi;
clear sizehalf;

%------------------------------------------------------------%
%--- Generation of final DAPI mask --------------------------%
%------------------------------------------------------------%

I6_dapi = bwlabel( (I5_dapi > 0) ); 
maxlabel = max(max(I6_dapi));
featsize = [];
for i = 1 : maxlabel
  featsize = [featsize, sum(I6_dapi(:) == i)];
end
[featsize, featind] = sortrows(featsize');
featlist = [featsize, featind];
%--- ordering of features within I5_dapi by increasing size

clear I5_dapi;
clear featsize; 
clear featind;
clear maxlabel;

I7_dapi = double(zeros(dim01, dim02)); 

if size(featlist,1) >= 1 && featlist(end,1) > fill_dapi
I7_dapi( (I6_dapi == featlist(end,2)) ) = 1;
end
%--- takes over the largest feature 
%--- if its size is at least fill_dapi px

if size(featlist,1) >= 2 && featlist((end-1),1) > fill_dapi 
I7_dapi( (I6_dapi == featlist( (end-1),2)) ) = 1;
end
%--- takes over the second largest feature
%--- if its size is at least fill_dapi px

if size(featlist,1) >= 3 && featlist((end-2),1) > fill_dapi 
I7_dapi( (I6_dapi == featlist( (end-2),2)) ) = 1;
end
%--- takes over the third largest feature
%--- if its size is at least fill_dapi px

if size(featlist,1) >= 4 && featlist((end-3),1) > fill_dapi 
I7_dapi( (I6_dapi == featlist( (end-3),2)) ) = 1;
end
%--- takes over the fourth largest feature
%--- if its size is at least fill_dapi px

if size(featlist,1) >= 5 && featlist((end-4),1) > fill_dapi 
I7_dapi( (I6_dapi == featlist( (end-4),2)) ) = 1;
end
%--- takes over the fifth largest feature
%--- if its size is at least fill_dapi px

if size(featlist,1) >= 6 && featlist((end-5),1) > fill_dapi 
I7_dapi( (I6_dapi == featlist( (end-5),2)) ) = 1;
end
%--- takes over the sixth largest feature
%--- if its size is at least fill_dapi px

clear I6_dapi;
clear featlist;

I7_dapi = 1 - I7_dapi;
%--- inversion of the mask

I8_dapi = bwlabel( (I7_dapi > 0) );
maxlabel = max(max(I8_dapi));
for i = 1 : maxlabel
if (sum(I8_dapi(:) == i) <= fill_dapi)
  I7_dapi(I8_dapi == i) = 0;
end, end
%--- filling of lacunae <= fill_dapi px

I7_dapi = 1 - I7_dapi;
%--- re-inversion of the mask

imwrite(I7_dapi, name_dapi_maske);
%--- saving of the final DAPI mask

clear I8_dapi;
clear dim01;
clear dim02;
clear maxlabel;

%------------------------------------------------------------%
%--- Analysis of the final DAPI mask ------------------------%
%------------------------------------------------------------%

%--- assumes that pixel_area is given in \mu m^2

dapipix = sum(sum(I7_dapi> 0));
schnittflaeche_dapi = pixel_area * dapipix;
schnittflaeche_dapi = schnittflaeche_dapi / (10^6);
%--- output of evaluation area in mm^2

clear I7_dapi;
clear dapipix;

%------------------------------------------------------------%
%--- Entry into logfile -------------------------------------%
%------------------------------------------------------------%

fid = fopen(name_logfile, 'a');
fprintf(fid, 'Analysis of DAPI channel at: %s\n', datestr(now, 'dd.mm.yyyy HH:MM:SS'));
fprintf(fid, '%s\n', name_prozedur_dapimask);
fprintf(fid, '%s\n', version_prozedur_dapimask);
fprintf(fid, '\n');

fprintf(fid, 'Parameters used for generation of DAPI cartoon:\n');
fprintf(fid, '\n');
fprintf(fid, 'medpar_dapi = [%d,%d]\n', medpar_dapi, medpar_dapi);
fprintf(fid, 'regpar_dapi = %f\n', regpar_dapi);
fprintf(fid, 'regit_dapi = %d\n', regit_dapi);
fprintf(fid, '\n');

fprintf(fid, 'Parameters used for generation of DAPI mask:\n');
fprintf(fid, '\n');
fprintf(fid, 'size_dapi_quadrat = %d\n', size_dapi_quadrat);
fprintf(fid, 'schwelle_dapi = %f\n', schwelle_dapi);
fprintf(fid, 'fill_dapi = %d\n', fill_dapi);
fprintf(fid, 'pixel_area = %f\n', pixel_area);
fprintf(fid, '\n');

fprintf(fid, 'Resulting files:\n');
fprintf(fid, '\n');
fprintf(fid, '%s\n', name_dapi_cartoon);
fprintf(fid, '%s\n', name_dapi_maske);
fprintf(fid, '\n');

fprintf(fid, '* Masked evaluation area (mm^2):\n');
fprintf(fid, '\n');
fprintf(fid, '%1.8f\n', schnittflaeche_dapi);
fprintf(fid, '\n');

clear schnittflaeche_dapi;

fprintf(fid, 'Manual correction of DAPI mask: no\n');
fprintf(fid, 'If applicable, masked evaluation area after correction (mm^2): \n');
fprintf(fid, '\n');
fprintf(fid, 'n/a\n');
fprintf(fid, '\n');
fprintf(fid, '-----------------------------------------------------------\n');
fprintf(fid, '\n');

fclose(fid);

