%------------------------------------------------------------%
%--- License terms ------------------------------------------%
%------------------------------------------------------------%

%--- This software is distributed under CC0 1.0 Public Domain 
%--- Dedication. To the extent possible under law, the authors
%--- (Marcus Wagner et al.) have dedicated all copyright and 
%--- related and neighboring rights to this software to the 
%--- public domain worldwide. This software is distributed 
%--- without any warranty. You should have received a copy of the 
%--- CC0 1.0 Universal Public Domain Dedication along with this 
%--- software. If not, write to Creative Commons, PO Box 1866, 
%--- Mountain View, CA 94042, USA.

%------------------------------------------------------------%
%--- Estimation of cell core number within DAPI channel -----%
%------------------------------------------------------------%

%--- assumes that flag_dapi == 1

name_prozedur_dapiestim = 'rofsegm_public_step_03b_dapiestim.m';
version_prozedur_dapiestim = 'version_2019_04_12';


%------------------------------------------------------------%
%--- Import of images ---------------------------------------%
%------------------------------------------------------------%

I1_dapi = double(imread( name_dapi_maske ))/255;
%--- DAPI mask

I2_dapi = double(imread( name_dapi_cartoon ))/255;
%--- DAPI cartoon


%------------------------------------------------------------%
%--- Estimation of cell core number within the cartoon ------% 
%------------------------------------------------------------%

I2_dapi = I2_dapi .* I1_dapi;
%--- masking of the cartoon

I2_dapi = I2_dapi >= schwelle_dapi;
%--- binarization fo the cartoon with the same threshold
%--- as in the generation of the DAPI mask

schnittflaeche_kern = 100;
%--- this estimate is based on a pixel area 
%--- of 0.454 * 0.454 \mu m^2

schaetzung_kernzahl = sum(sum(I2_dapi)) / schnittflaeche_kern;
%--- this estimate is based on a pixel area 
%--- of 0.454 * 0.454 \mu m^2
%--- and a cell core diameter of 5 \mu m, thus resulting in
%--- a section area of about 100 px

schaetzung_kernzahl = floor(schaetzung_kernzahl);
%--- rounding down

clear I1_dapi;
clear I2_dapi;


%------------------------------------------------------------%
%--- Entry into logfile -------------------------------------%
%------------------------------------------------------------%

fid = fopen(name_logfile, 'a');
fprintf(fid, 'Analysis of DAPI channel at: %s\n', datestr(now, 'dd.mm.yyyy HH:MM:SS'));
fprintf(fid, '%s\n', name_prozedur_dapiestim);
fprintf(fid, '%s\n', version_prozedur_dapiestim);
fprintf(fid, '\n');

fprintf(fid, 'Parameters used for estimation of cell core number:\n');
fprintf(fid, '\n');
fprintf(fid, 'schwelle_dapi = %f\n', schwelle_dapi);
fprintf(fid, 'schnittflaeche_kern = %d\n', schnittflaeche_kern);
fprintf(fid, '\n');

fprintf(fid, '* Estimated number of cell cores within masked evaluation area (#):\n');
fprintf(fid, '\n');
fprintf(fid, '%d\n', schaetzung_kernzahl);
fprintf(fid, '\n');
fprintf(fid, '-----------------------------------------------------------\n');
fprintf(fid, '\n');

fclose(fid);

clear schaetzung_kernzahl;
