%------------------------------------------------------------%
%--- License terms ------------------------------------------%
%------------------------------------------------------------%

%--- This software is distributed under CC0 1.0 Public Domain 
%--- Dedication. To the extent possible under law, the authors
%--- (Marcus Wagner et al.) have dedicated all copyright and 
%--- related and neighboring rights to this software to the 
%--- public domain worldwide. This software is distributed 
%--- without any warranty. You should have received a copy of the 
%--- CC0 1.0 Universal Public Domain Dedication along with this 
%--- software. If not, write to Creative Commons, PO Box 1866, 
%--- Mountain View, CA 94042, USA.

%------------------------------------------------------------%
%--- Segmentation of CD163/555nm channel --------------------%
%------------------------------------------------------------%

%--- assumes that flag_555 == 1

name_prozedur_555 = 'rofsegm_public_step_05_channel_555.m';
version_prozedur_555 = 'version_2019_04_12';

%---------------------------------------------------------- -%
%--- Import of images ---------------------------------------%
%------------------------------------------------------------%


I0_555 = double(imread( name_555_ori ))/255;
%--- original image for CD163/555nm channel

dim01 = size(I0_555, 1); dim02 = size(I0_555, 2);

if flag_dapi == 1
I0_dapimask = double(imread( name_dapi_maske ))/255;
%--- DAPI mask
else
I0_dapimask = double(ones(dim01, dim02));
%--- complete image becomes evaluation area 
%--- if no DAPI mask available,
end

%------------------------------------------------------------%
%--- Median filtering and TV regularization -----------------%
%------------------------------------------------------------%

I1_555 = medfilt2(I0_555, [medpar_555,medpar_555], 'symmetric');
%--- Median filtering of I0_555 with medpar_555

I2_555 = I0_555 - I1_555;

clear I1_555;

I2_555 = I2_555 .* (I2_555 >= 0);
%--- cutting of possible negative values

I3_555 = tgv1_l2_2D_pd(I2_555, I2_555, regpar_555, regit_555);
%--- TV regularization, requires function tgv1_l2_2D
display(' ');

clear I2_555;

imwrite(I3_555, name_555_cartoon);
%--- saving of cartoon

%------------------------------------------------------------%
%--- Initialization of result variables ---------------------%
%------------------------------------------------------------%

result = 0; schaetzung = 1;

Phagenflaeche = double(zeros(dim01,dim02));
%--- mask for entering the segmented macrophages 

Phagenflaeche_conv = double(zeros(dim01,dim02));
%--- maske for entering the convex hulls 
%--- of the segmented macrophages

Phagenliste = [1, 0, 0, 0, 0, 0, 0, 0];
%--- list with the following entries:
%--- 1) number of the feature in Phagenflaeche
%--- 2) grey value, at which the feature has been detected 
%--- 3) feature size in px
%--- 4) mean feature brightness in I0_488
%--- 5) st. dev. of feature brightness in I0_488
%--- 6) ratio between size of convex hull and feature size itself
%--- 7) ratio of axes in covering ellipsoid
%--- 8) feature perimeter in px

%--- only for features satisfying all criteria
%--- new row will be appended

%------------------------------------------------------------%
%--- Preparatory steps for segmentation ---------------------%
%------------------------------------------------------------%

I3_555_reshaped = reshape(I3_555, [1, dim01 * dim02]);
m3 = mean(I3_555_reshaped);
s3 = std(I3_555_reshaped, 1);

schwelle_unten = ceil(m3 * 255);
%--- mean grey level in I3_555, rounded up

schwelle_oben = ceil(max(I3_555_reshaped) * 255);
%--- maximal grey level in I3_488, <= 255

Imask = I3_555 .* (I3_555 >= schwelle_unten/255);
Imask = Imask .* I0_dapimask;
%--- initialization of a mask for progressive segmentation 
%--- restriction to masked evaluation area just here

clear I3_555;
clear I3_555_reshaped;
clear I0_dapimask;

%------------------------------------------------------------%
%--- The segmentation itself --------------------------------%
%------------------------------------------------------------%

%%% =================== Loop 1 =========================== %%%

for laufindex = schwelle_unten : schwelle_oben

%--- segmentation of I3_555 runs from grey level "schwelle_unten" 
%--- until grey level "schwelle_oben"

Iseg1 = (Imask >= laufindex/255);
Imask = Imask .* Iseg1;
Iseg2 = bwlabel(Iseg1);
maxlabel = max(Iseg2(:));

clear Iseg1;

featurelist = double(zeros(maxlabel,9)); 
%--- temporary list of features with the following entries:
%--- 1) sequential number
%--- 2) grey level at which the feature has been detected 
%--- 3) feature size in px
%--- 4) mean feature brightness in I0_488
%--- 5) st. dev. of feature brightness in I0_488
%--- 6) ratio between size of convex hull and feature size itself
%--- 7) ratio of axes in covering ellipsoid
%--- 8) feature perimeter in px
%--- 9) estimation of maximal number of macrophages contained 

for j = 1 : maxlabel
  featurelist(j,1) = j;
  featurelist(j,2) = laufindex;
  Itemp = (Iseg2 == j);
  featurelist(j,3) = sum(Itemp(:) == 1);
  Itemp_reshaped = I0_555 .* Itemp;
  Itemp_reshaped = reshape(Itemp_reshaped, [1, dim01 * dim02]);
  Itemp_reshaped = Itemp_reshaped(Itemp_reshaped(:) > 0);
  featurelist(j,4) = mean(Itemp_reshaped);
  featurelist(j,5) = std(Itemp_reshaped, 1);
  Itemp_conv = bwconvhull(Itemp, 'union');
  featurelist(j,6) = sum(Itemp_conv(:) ==1) / featurelist(j,3);
  properties = regionprops(Itemp, 'MajorAxisLength', 'MinorAxisLength', 'Perimeter');
  featurelist(j,7) = [properties.MajorAxisLength] / [properties.MinorAxisLength];
  featurelist(j,8) = [properties.Perimeter];
  featurelist(j,9) = floor(featurelist(j,3) / feature_555_min);
end

clear Itemp;
clear Itemp_reshaped;
clear Itemp_conv;

featurelist = sortrows(featurelist, 3);
%--- sorting the temporary list by feature size

%%% =================== Loop 2 =========================== %%%

%--- detection of macrophages at given grey level

for j = 1 : maxlabel

  if featurelist(j,9) == 0
%--- size too small 
    Imask(Iseg2(:) == featurelist(j,1)) = 0; 
%--- delete the feature from Imask

  elseif featurelist(j,9) == 1
%--- presumed single macrophage
    if (featurelist(j,6) * 100 - 100 >= konvpar_555) && (featurelist(j,7) <= excentpar_555) && (featurelist(j,8) / ( 2 * sqrt( pi * featurelist(j,3) )) <= bdrypar_555)
%--- all of 3 geometrical criteria satisfied



      index_temp = Phagenliste(end,1);
      Phagenliste(end, 2) = featurelist(j,2);
      Phagenliste(end,3) = featurelist(j,3);
      Phagenliste(end,4) = featurelist(j,4);
      Phagenliste(end,5) = featurelist(j,5);
      Phagenliste(end,6) = featurelist(j,6);
      Phagenliste(end,7) = featurelist(j,7);
      Phagenliste(end,8) = featurelist(j,8);
%--- entry of feature properties into Phagenliste

      Phagenliste = [ Phagenliste; [(index_temp + 1), 0, 0, 0, 0, 0, 0, 0] ];
%--- appending a new row to Phagenliste

      Phagenflaeche(Iseg2(:) == featurelist(j,1)) = Phagenliste(end,1);
%--- entry of the feature into Phagenflaeche using its 
%--- sequential number from Phagenliste 

      Iseg_temp = bwconvhull( (Phagenflaeche == Phagenliste(end,1)), 'union');
      Phagenflaeche_conv(Iseg_temp == 1) = Phagenliste(end,1);
%--- entry of the convex hull of the feature into  
%--- Phagenflaeche_conv using its sequential number 
%--- from Phagenliste 

    else

      featurelist(j,9) = 0;
%--- labeling as a feature, which will not be counted 
%--- as a macrophage 

    end

    Imask(Iseg2(:) == featurelist(j,1)) = 0; 
%--- independently on its interpretation, the feature will be
%--- deleted from Imask 


  elseif featurelist(j,9) >= 2 && featurelist(j,3) <= feature_555_max && featurelist(j,7) > excentpar_555
%--- larger feature <= maximal size but too elongated 

  featurelist(j,9) = 0;
%--- labeling as a feature, which will not be counted 
%--- as a macrophage 

    Imask(Iseg2(:) == featurelist(j,1)) = 0; 
%--- delete the feature from Imask

  elseif featurelist(j,9) >= 2 && featurelist(j,3) <= feature_555_max && (featurelist(j,6) * 100 - 100 >= konvpar_555) && (featurelist(j,7) <= excentpar_555) && (featurelist(j,8) / ( 2 * sqrt( pi * featurelist(j,3) )) <= bdrypar_555)
%--- larger feature <= maximal size
%--- all of 3 geometrical criteria satisfied

    index_temp = Phagenliste(end,1);
    Phagenliste(end, 2) = featurelist(j,2);
    Phagenliste(end,3) = featurelist(j,3);
    Phagenliste(end,4) = featurelist(j,4);
    Phagenliste(end,5) = featurelist(j,5);
    Phagenliste(end,6) = featurelist(j,6);
    Phagenliste(end,7) = featurelist(j,7);
    Phagenliste(end,8) = featurelist(j,8);
%--- entry of feature properties into Phagenliste

    Phagenliste = [ Phagenliste; [(index_temp + 1), 0, 0, 0, 0, 0, 0, 0] ];
%--- appending a new row to Phagenliste

    Phagenflaeche(Iseg2(:) == featurelist(j,1)) = Phagenliste(end,1);
%--- entry of the feature into Phagenflaeche using its 
%--- sequential number from Phagenliste 

    Iseg_temp = bwconvhull( (Phagenflaeche == Phagenliste(end,1)), 'union');
    Phagenflaeche_conv(Iseg_temp == 1) = Phagenliste(end,1);
%--- entry of the convex hull of the feature into  
%--- Phagenflaeche_conv using its sequential number 
%--- from Phagenliste 

    Imask(Iseg2(:) == featurelist(j,1)) = 0; 
%--- delete the feature from Imask

    featurelist(j,9) = 1;
%--- relabel the feature as a single macrophage

   end

%%% =================== End of Loop 2 ==================== %%%

end

result = result + sum( featurelist(:,9) .* (featurelist(:,9) == 1) );
schaetzung = result + sum( featurelist(:,9) .* (featurelist(:,9) >= 2) );
%--- estimate the segmentation result


%%% =================== End of Loop 1 ==================== %%%

end

display(result);
display(' ');

clear I0_555;
clear Imask;
clear Iseg2;
clear Iseg_temp;
clear Phagenliste;
clear featurelist;

%------------------------------------------------------------%
%--- Saving the segmentation mask, feature counting ---------%
%------------------------------------------------------------%

imwrite(Phagenflaeche, name_555_segment);
imwrite(Phagenflaeche_conv, name_555_convhull);
%--- saving the feature masks
%--- (which are already restricted to the evaluation area)

Phagenflaeche = (Phagenflaeche > 0);
Phagenflaeche = bwlabel(Phagenflaeche);

result_mask = max(max(Phagenflaeche));
%--- number of macrophages within evaluation area

clear maxlabel;
clear Phagenflaeche;
clear Phagenflaeche_conv;

%------------------------------------------------------------%
%--- Entry into logfile -------------------------------------%
%------------------------------------------------------------%

fid = fopen(name_logfile, 'a');
fprintf(fid, 'Analysis of the CD163/555nm channel at: %s\n', datestr(now, 'dd.mm.yyyy HH:MM:SS'));
fprintf(fid, '%s\n', name_prozedur_555);
fprintf(fid, '%s\n', version_prozedur_555);
fprintf(fid, '\n');

fprintf(fid, 'parameters used for generation of CD163/555nm cartoon:\n');
fprintf(fid, '\n');
fprintf(fid, 'medpar_555 = [%d,%d]\n', medpar_555, medpar_555);
fprintf(fid, 'regpar_555 = %f\n', regpar_555);
fprintf(fid, 'regit_555 = %d\n', regit_555);
fprintf(fid, '\n');

fprintf(fid, 'Parameters used for segmentation of CD163/555nm channel:\n');
fprintf(fid, '\n');
fprintf(fid, 'feature_555_min = %d\n', feature_555_min);
fprintf(fid, 'feature_555_max = %d\n', feature_555_max);
fprintf(fid, 'konvpar_555 = %f\n', konvpar_555);
fprintf(fid, 'excentpar_555 = %f\n', excentpar_555);
fprintf(fid, 'bdrypar_555 = %f\n', bdrypar_555);
fprintf(fid, '\n');

fprintf(fid, 'Resulting files:\n');
fprintf(fid, '\n');
fprintf(fid, '%s\n', name_555_cartoon);
fprintf(fid, '%s\n', name_555_segment);
fprintf(fid, '%s\n', name_555_convhull);
fprintf(fid, '\n');

fprintf(fid, '* Number of CD163-positive macrophages within masked evaluation area (#):\n');
fprintf(fid, '\n');
fprintf(fid, '%d\n', result_mask);
fprintf(fid, '\n');
fprintf(fid, '-----------------------------------------------------------\n');
fprintf(fid, '\n');
fclose(fid);

clear result;
clear result_mask;




