%------------------------------------------------------------%
%--- License terms ------------------------------------------%
%------------------------------------------------------------%

%--- This software is distributed under CC0 1.0 Public Domain 
%--- Dedication. To the extent possible under law, the authors
%--- (Marcus Wagner et al.) have dedicated all copyright and 
%--- related and neighboring rights to this software to the 
%--- public domain worldwide. This software is distributed 
%--- without any warranty. You should have received a copy of the 
%--- CC0 1.0 Universal Public Domain Dedication along with this 
%--- software. If not, write to Creative Commons, PO Box 1866, 
%--- Mountain View, CA 94042, USA.

%------------------------------------------------------------%
%--- Recognition of double stainings ------------------------%
%------------------------------------------------------------%

%--- assumes that flag_488 == 1 & flag_555 ==1

name_prozedur_doppel = 'rofsegm_public_step_06_double.m';
version_prozedur_doppel = 'version_2019_04_12';

%------------------------------------------------------------%
%--- Import of images ---------------------------------------%
%------------------------------------------------------------%

I0_488_segm = double(imread( name_488_segment ))/255;
I0_488_conv = double(imread( name_488_convhull ))/255;
I0_555_segm = double(imread( name_555_segment ))/255;
I0_555_conv = double(imread( name_555_convhull ))/255;

dim01 = size(I0_488_segm, 1); dim02 = size(I0_488_segm, 2);

if flag_dapi == 1
I0_dapimask = double(imread( name_dapi_maske ))/255;
%--- DAPI mask
else
I0_dapimask = double(ones(dim01, dim02));
%--- complete image becomes evaluation area 
%--- if no DAPI mask available,
end


%------------------------------------------------------------%
%--- Recognition of double-stained CD14-positive phages -----%
%------------------------------------------------------------%

I1_488 = bwlabel(I0_488_segm);
maxlabel = max(max(I1_488));

I1_488_mask = I1_488 .* I0_dapimask;

clear I0_488_segm;

for i = 1 : maxlabel
if sum(sum(I1_488 == i)) > sum(sum(I1_488_mask == i))
I1_488(I1_488 == i) = 0;
end, end
%--- macrophages cutted by the mask will be deleted 

clear maxlabel;

I1_488 = (I1_488 > 0);
I1_488 = bwlabel(I1_488);
maxlabel = max(max(I1_488));

I2_488 = double(zeros(dim01, dim02));

for i = 1 : maxlabel

Itemp = (I1_488 == i);
Itemp_conv = bwconvhull(Itemp, 'union');
%--- convex hull of current feature in temporary image
fltemp = sum(sum( Itemp_conv));
%--- determine its size in px
Itemp_doppel = Itemp_conv .* I0_555_conv;
%--- intersect it with the 555_convhull image
fltemp_doppel = sum(sum( Itemp_doppel ));
%--- determine the intersection size
ratio_555_488 = fltemp_doppel / fltemp;
if ratio_555_488 >= prozent_CD14_555_gefaerbt
I2_488(I1_488 == i) = 1;
%--- check the ratio of both; if it is large enough, 
%--- save the feature into double-staining mask
end, end

%------------------------------------------------------------%
%--- Saving the result, counting of macrophages -------------%
%------------------------------------------------------------%

imwrite(I2_488, name_488_doppel);

result_488_doppel = max(max(bwlabel(I2_488)));
%--- number of double-stained CD14-positive macrophages

clear I0_555_conv;
clear I1_488;
clear I1_488_mask;
clear Itemp;
clear Itemp_conv;
clear Itemp_doppel;

%------------------------------------------------------------%
%--- Recognition of double-stained CD163-positive phages ----%
%------------------------------------------------------------%

I1_555 = bwlabel(I0_555_segm);
maxlabel = max(max(I1_555));

I1_555_mask = I1_555 .* I0_dapimask;

clear I0_555_segm;
clear I0_dapimask;

for i = 1 : maxlabel
if sum(sum(I1_555 == i)) > sum(sum(I1_555_mask ==i))
I1_555(I1_555 == i) = 0;
end, end
%--- macrophages cutted by the mask will be deleted 

clear maxlabel;

I1_555 = (I1_555 > 0);
I1_555 = bwlabel(I1_555);
maxlabel = max(max(I1_555));

I2_555 = double(zeros(dim01, dim02));

for i = 1 : maxlabel

Itemp = (I1_555 == i);
Itemp_conv = bwconvhull(Itemp, 'union');
%--- convex hull of current feature in temporary image
fltemp = sum(sum( Itemp_conv));
%--- determine its size in px
Itemp_doppel = Itemp_conv .* I0_488_conv;
%--- intersect it with the 488_convhull image
fltemp_doppel = sum(sum( Itemp_doppel ));
%--- determine the intersection size
ratio_488_555 = fltemp_doppel / fltemp;
if ratio_488_555 >= prozent_CD163_488_gefaerbt
I2_555(I1_555 == i) = 1;
%--- check the ratio of both; if it is large enough, 
%--- save the feature into double-staining mask
end, end

%------------------------------------------------------------%
%--- Saving the result, counting of macrophages -------------%
%------------------------------------------------------------%


imwrite(I2_555, name_555_doppel);

result_555_doppel = max(max(bwlabel(I2_555)));
%--- number of double-stained CD163-positive macrophages

clear I0_488_conv;
clear I1_555;
clear I1_555_mask;
clear Itemp;
clear Itemp_conv;
clear Itemp_doppel;


%------------------------------------------------------------%
%--- Entry into logfile -------------------------------------%
%------------------------------------------------------------%

fid = fopen(name_logfile, 'a');
fprintf(fid, 'Recognition of double stainings at: %s\n', datestr(now, 'dd.mm.yyyy HH:MM:SS'));
fprintf(fid, '%s\n', name_prozedur_doppel);
fprintf(fid, '%s\n', version_prozedur_doppel);
fprintf(fid, '\n');

fprintf(fid, 'Parameter used for detection of double-stained CD14-positive macrophages:\n');
fprintf(fid, '\n');
fprintf(fid, 'prozent_CD14_555_gefaerbt = %f\n', prozent_CD14_555_gefaerbt);
fprintf(fid, '\n');

fprintf(fid, 'Parameter used for detection of double-stained CD163-positive macrophages:\n');
fprintf(fid, '\n');
fprintf(fid, 'prozent_CD163_488_gefaerbt = %f\n', prozent_CD163_488_gefaerbt);
fprintf(fid, '\n');


fprintf(fid, 'Resulting files:\n');
fprintf(fid, '\n');
fprintf(fid, '%s\n', name_488_doppel);
fprintf(fid, '%s\n', name_555_doppel);
fprintf(fid, '\n');

fprintf(fid, '* Number of double-stained CD14-positive macrophages within masked evaluation area (#):\n');
fprintf(fid, '\n');
fprintf(fid, '%d\n', result_488_doppel);
fprintf(fid, '\n');

fprintf(fid, '* Number of double-stained CD163-positive macrophages within masked evaluation area (#):\n');
fprintf(fid, '\n');
fprintf(fid, '%d\n', result_555_doppel);
fprintf(fid, '\n');

fprintf(fid, '-----------------------------------------------------------\n');
fprintf(fid, '\n');
fclose(fid);

clear result_488_doppel;
clear result_555_doppel;



